import java.awt.Color;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.MouseInfo;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.awt.image.BufferedImage;
import java.io.File;
import java.io.IOException;
import java.util.Date;
import java.util.Timer;
import java.util.TimerTask;

import javax.imageio.ImageIO;
import javax.swing.JFrame;
import javax.swing.JPanel;

public class Animation extends JPanel {
    static final int WIDTH = 600, HEIGHT = 400;
    // image de fond
    BufferedImage img_back; 
    // image de la bille en .gif, a fond transparent
    BufferedImage img_ball;
    // coordonnees courantes du coin superieur gauche de img_ball
    int x;           
    int y;
    // coordonnees de depart
    int xc;
    int yc;
    // maximum des coordonnes pour le rebond.
    int max_x;
    int max_y;
    // increments des coordonnees
    int dx;            
    int dy;
    // flag indiquant si la bille est en cours d'animation
    boolean moving;
    // planificateur de taches
    Timer t;
    // prochaine date de rafraichissement du panneau
    Date dr;

    public Animation () {
	this.setPreferredSize(new Dimension(WIDTH, HEIGHT));
	createBackgroundImage ();
	loadForegroundImage (); 
	// creation du planificateur de taches
	t = new Timer ();
	buildKeyListener ();
    }

    /* la methode est surchargee pour :
     * (1) dessiner le fond
     * (2) dessiner la bille
     */
    protected void paintComponent (Graphics g) {
	g.drawImage (img_back, 0, 0, null);  
	g.drawImage (img_ball, x, y, null);  
    }

    // creation de l'image de fond
    void createBackgroundImage () {
	this.img_back = new BufferedImage (WIDTH, HEIGHT, BufferedImage.TYPE_INT_RGB);
	Graphics g_back = img_back.getGraphics ();
	g_back.setColor (new Color (41, 98, 41));
	g_back.fillRect (0, 0, WIDTH, HEIGHT);
    }

    /* chargement de l'image de la bille, au format gif,
     * donc a fond transparent - ce qui evite que l'image
     * ne recouvre le fond hors des limite de la bille.
     */
    void loadForegroundImage () {
	try {
	    img_ball = ImageIO.read (new File ("bille.gif"));
	} catch (IOException e) {
	    e.printStackTrace ();
	    System.exit (1);
	}
	int width  = img_ball.getWidth ();
	int height = img_ball.getHeight ();
	this.max_x = WIDTH  - width;
	this.max_y = HEIGHT - height;
	// centrage de l'image
	xc = WIDTH  / 2 - (width  / 2);
	yc = HEIGHT / 2 - (height / 2);
	x = xc;
	y = yc;
    }

    // ensemble des codes de touches geres par l'application
    static int[] validCodes = {
	KeyEvent.VK_LEFT, 
	KeyEvent.VK_RIGHT, 
	KeyEvent.VK_UP, 
	KeyEvent.VK_DOWN
    };
    /* increments associes aux touches pour le demarrage du
     * mouvement de la bille.
     */
    static int[] all_dx = {-1, 1, 0, 0};
    static int[] all_dy = {0, 0, -1, 1};

    void buildKeyListener () {
	/* creation de l'ecouteur d'evenements-clavier 
	 *  qui ecoutera le panneau.
	 */
	KeyListener kl = new KeyListener () {
		public void keyPressed (KeyEvent e) {
		    /* si une animation est deja en cours,
		     * l'ecouteur ne fait rien.
		     */
		    if (moving) {
			return;
		    }
		    /* sinon, il cherche dans le tableau des codes
		     * de touches le code de la touche pressee.
		     */
		    int i;
		    for (i = 0; i < 4; i++) {
			// si le code est dans le tableau,
			if (e.getKeyCode () == validCodes[i]) {
			    // on initialise les increments
			    dx = all_dx[i];
			    dy = all_dy[i];
			    moving = true;
			    /* La methode update () renvoie une "tache"
			     * (TimerTask). on planifie l'invocation de sa 
			     * methode run () dans 5 millisecondes.
			     */
			    dr = new Date (System.currentTimeMillis () + 5);
			    t.schedule (update (), dr);
			    return;
			}
		    }

		}
		public void keyReleased (KeyEvent e) {}
		public void keyTyped (KeyEvent e) {}
	    }; 
	this.setFocusable (true);
	this.addKeyListener (kl);
    }

    /* methode renvoyant une tache pour le planificateur de 
     * tache t - sa methode run () sera invoquee a la
     * date dr.
     */
    TimerTask update () {
	// creation de la tache, de classe anonyme
	return new TimerTask () {
	    /* implementation de la methode invoquee a la date
	     * d'execution de cette tache.
	     */
	    public void run () {
		/* HACK : necessaire pour eviter que l'animation
		 * ne soit saccadee (le resultat n'est pas toujours
		 * parfait, mais il faudra s'en contenter)
		 */
		MouseInfo.getPointerInfo ();
		// simple mise a jour des coordonnes et increments
		x += dx;
		y += dy;
		if (x < 0) {
		    dx = -dx;
		    x = 0;
		} else if (x > max_x) {
		    dx = -dx;
		    x = max_x;
		}
		if (y < 0) {
		    dy = -dy;
		    y = 0;
		} else if (y > max_y) {
		    dy = -dy;
		    y = max_y;
		}
		// rafraichissement du panneau
		repaint ();	
		/* si la bille a rejoint ses coordonnees d'origine,
		 * on interrompt l'animation.
		 */
		if (x == xc && y == yc) {
		    moving = false;	
		    return;
		}
		/* sinon on planifie la prochaine tache dans 5 millisecondes 
		 * - i.e. l'animation se poursuit.
		 */
		dr = new Date (dr.getTime () + 5);
		t.schedule(update (), dr);
	    }
	};
    }

    public static void main (String[] args) {
	JFrame frame = new JFrame ();
	frame.setDefaultCloseOperation (JFrame.EXIT_ON_CLOSE);
	frame.add (new Animation ());
	frame.pack ();
	frame.setVisible (true);
    }

}
